﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Vintagestory.API.Client;
using Vintagestory.API.Server;
using Vintagestory.API.Common;
using Vintagestory.GameContent;
using Vintagestory.API.Config;
using Vintagestory.API.MathTools;
using Vintagestory.API.Common.Entities;
using Vintagestory.API.Datastructures;



namespace movemod.src
{
    public class MoveMod : ModSystem
    {
        ICoreClientAPI capi;

        // Data Fields
        public int maxChargeJump = 100;
        public int chargeJump = 0;

        public int maxDashCountdown = 10; // Ticks (50ms) to reset dash & slide ability --- min 6 !
        public int dashCountdown = 0;

        public static float maxStamina = 100; // Maximum stamina
        public static float stamina = maxStamina;
        public int maxStaminaRegenDelay = 30; // Ticks (50ms) until stamina-regeneration
        public int staminaRegenDelay = 0;  
        public int staminaRegenValue = 5; // Stamina regen per tick
        
        public int dashTime = 5;  
        public int slideTime = 10; 
        public float drainStaminaDash = (float)(maxStamina * 0.33333); // Stamina drain for dash / slide / walljump

        public float drainStaminaClimbing = 2f; // Stamina drain while climbing
        public float drainStaminaChargeJump = (float)(maxStamina * 0.03); // Stamina drain for charge jump / 10 ( => 0.04 = 40% )
        public float drainStaminaHanging = 0.4f; // Stamina drain while hanging
        public float drainStaminaIdleModificator = 0.2f; // Stamina drain modificator while hanging / climbing idle (not moving) in %

        public int spaceDoubleTap = 0;
        public static bool spacePressed = false;

        public static bool playerIsClimbing = false;
        public bool isCrawling = false;

        bool staminaRegenIsDelayed = false;
        bool isClimbable = false;
        bool isHanging = false;
        bool motionStopedAfterDash = true;

        // Hotkey bools
        public static bool toggleStandup = false;
        public static bool toggleCrawl = false;
        
        bool slotChanged = false;

        // SERVERSIDE =============================================================================
        public override void StartServerSide(ICoreServerAPI api)
        {
            base.StartServerSide(api);
        }

        // CLIENTSIDE =============================================================================
        public override void StartClientSide(ICoreClientAPI api)
        {

            api.Event.PlayerEntitySpawn += entity =>
            entity.Entity.GetBehavior<EntityBehaviorControlledPhysics>().stepHeight = 0.9F;

            api.Event.PlayerEntitySpawn += entity =>
            entity.Entity.Properties.CanClimbAnywhere = false;

            api.Event.PlayerEntitySpawn += entity =>
            entity.Entity.SetHitbox(0.5f, 2f);

            // Hotkey Events
            // HotKey Stop crawling
            api.Input.RegisterHotKey("standup", "Stand Up from crawling", GlKeys.Space, HotkeyType.CharacterControls);
            api.Input.SetHotKeyHandler("standup", StandUp);

            // HotKey Start Crawl
            api.Input.RegisterHotKey("startcrawl", "Crawl on the floor using Sneak + Hotkey", GlKeys.R, HotkeyType.CharacterControls);
            api.Input.SetHotKeyHandler("startcrawl", StartCrawl);


            // Game Time Tick Listener 10ms ========================================================
            api.Event.RegisterGameTickListener(dt1 =>
            {
                var player = api.World.Player;

                // Stop Motion Vec AfterDash
                if (dashTime == 5 && motionStopedAfterDash == false && player.Entity.OnGround)
                {
                    Vec3d motionZero = new Vec3d(0, 0, 0);
                    player.Entity.Pos.Motion = motionZero;
                    motionStopedAfterDash = true;
                }

                // Prevent player from climbing down
                if (player.Entity.Controls.IsClimbing && !isClimbable)
                {
                    player.Entity.Controls.Sneak = false;
                    api.Input.InWorldAction += Input_InWorldAction3;
                }
                else if (!isCrawling)
                {
                    api.Input.InWorldAction += Input_InWorldAction1;
                }          

            }, 10);

            // Game Time Tick Listener 50ms ========================================================
            api.Event.RegisterGameTickListener(dt =>
            {
                var player = api.World.Player;

                // DEBUG ==================
                //player.ShowChatNotification("Stamina " + stamina.ToString());
                //player.ShowChatNotification("Jumpforce " + chargeJump.ToString());
                //player.ShowChatNotification("Dash Timer: " + dashCountdown.ToString());
                //player.ShowChatNotification("Activity State: " + player.Entity.CurrentControls.ToString());
                //player.ShowChatNotification("Activity isClimbing?: " + player.Entity.Controls.IsClimbing.ToString());
                //player.ShowChatNotification("Activity Jump?: " + player.Entity.Controls.Jump.ToString());
                //player.ShowChatNotification("Block at PlayerPos cimbable?: " + isClimbable.ToString());

                //player.ShowChatNotification("Player on ground?: " + player.Entity.OnGround.ToString());
                //player.ShowChatNotification("Dash-Time: " + slideTime.ToString());
                //player.ShowChatNotification("Motion Stopped?: " + motionStopedAfterDash.ToString());
                // DEBUG ==================

                // Experimental

                //// Gear Change Event
                //player.Entity.GearInventory.SlotModified += GearInventory_SlotModified;
                //if (slotChanged)
                //{
                //    slotChanged = false;
                //    player.ShowChatNotification("Item Slot changed");
                //}

                //// Check for specific item in active slot
                //Item checkItem = api.World.GetItem(new AssetLocation("bow-long"));
                //if (!player.InventoryManager.ActiveHotbarSlot.Empty)
                //{
                //    if (player.InventoryManager.ActiveHotbarSlot.Itemstack.Item == checkItem)
                //    {
                //        player.ShowChatNotification("Item in hand is a bow-long");
                //    }
                //}
                //else           // null Exception catch
                //{
                //    player.ShowChatNotification("No item in hand");
                //}

                ////Check for specific wearable
                //var gearInv = player.Entity.GearInventory;
                //Item checkItemWearable = api.World.GetItem(new AssetLocation("game:clothes-head-silver-diadem"));

                //foreach (var slot in gearInv)
                //{
                //    ItemWearable item;
                //    if (slot.Empty || (item = slot.Itemstack.Collectible as ItemWearable) == null) continue; // null Exception catch
                //    if ((item = slot.Itemstack.Collectible as ItemWearable) == checkItemWearable)
                //    {
                //        player.ShowChatNotification("You are wearing the mysic item");
                //    }
                //}

                // Get block at playerPos
                Block checkBlockatPlayerPos = api.World.BlockAccessor.GetBlock((int)player.Entity.Pos.X, (int)player.Entity.Pos.Y, (int)player.Entity.Pos.Z);

                // Block above is air ?
                Block water = api.World.GetBlock(new AssetLocation("water"));
                Block air = api.World.GetBlock(new AssetLocation("air"));
                Block aboveBlock = api.World.BlockAccessor.GetBlock((int)player.Entity.Pos.X, (int)player.Entity.Pos.Y + 1, (int)player.Entity.Pos.Z);
                //player.ShowChatNotification("Block checked: " + aboveBlock.ToString());

                // Counter per Tick (Dash Timer)
                if (dashCountdown > 0)
                {
                    dashCountdown--;
                }

                // Counter per Tick (Stamina Regen-Delay)
                if (staminaRegenDelay > 0)
                {
                    staminaRegenDelay--;
                }

                //  0 Stamina Slow
                if (stamina < 1 && player.Entity.OnGround)
                {
                    Vintagestory.API.Config.GlobalConstants.OverallSpeedMultiplier = 0.75f;
                }
                else
                {   
                    Vintagestory.API.Config.GlobalConstants.OverallSpeedMultiplier = 1f;
                }

                // Hanging
                Block aboveBlock2 = api.World.BlockAccessor.GetBlock((int)player.Entity.Pos.X, (int)player.Entity.Pos.Y + 2, (int)player.Entity.Pos.Z);

                if (aboveBlock2 != air && stamina > 0 && !player.Entity.OnGround && player.Entity.Controls.Sneak && !player.Entity.Controls.TriesToMove && player.InventoryManager.ActiveHotbarSlot.Empty && isClimbable == false)
                {
                    Vec3d motionZero = new Vec3d(0, 0, 0);
                    player.Entity.Pos.Motion = motionZero;

                    player.Entity.Properties.Habitat = EnumHabitat.Air;
                    player.Entity.Properties.CanClimbAnywhere = false;
                    player.Entity.StopAnimation("hangingtop");
                    player.Entity.StartAnimation("hangingidle");
                    stamina -= (drainStaminaHanging * drainStaminaIdleModificator);
                    isHanging = true;
                }
                else if (aboveBlock2 != air && stamina > 0 && !player.Entity.OnGround && player.Entity.Controls.Sneak && player.Entity.Controls.TriesToMove && player.InventoryManager.ActiveHotbarSlot.Empty && isClimbable == false)
                {
                    player.Entity.Properties.Habitat = EnumHabitat.Air;
                    player.Entity.Properties.CanClimbAnywhere = false;
                    player.Entity.StopAnimation("hangingidle");
                    player.Entity.StartAnimation("hangingtop");
                    stamina -= drainStaminaHanging;
                    isHanging = true;
                }
                else if (aboveBlock2 != air && stamina > 0 && !player.Entity.OnGround && player.Entity.Controls.IsClimbing && player.InventoryManager.ActiveHotbarSlot.Empty && isClimbable == false)
                {
                    player.Entity.Properties.Habitat = EnumHabitat.Air;
                    player.Entity.Properties.CanClimbAnywhere = false;
                    player.Entity.StopAnimation("hangingidle");
                    player.Entity.StartAnimation("hangingtop");
                    stamina -= drainStaminaHanging;
                    isHanging = true;
                }
                else
                {
                        player.Entity.Properties.Habitat = EnumHabitat.Land;
                        player.Entity.StopAnimation("hangingtop");
                        player.Entity.StopAnimation("hangingidle");
                        isHanging = false;
                }          

                // Crawl funtion hitbox and animation
                if (isCrawling)
                {
                    player.Entity.Properties.CanClimbAnywhere = false;
                    player.Entity.StartAnimation("liegen");
                    player.Entity.SetHitbox(0.5f, 0.5f);
                    player.Entity.Properties.SetEyeHeight(0.6);

                    if (player.Entity.Controls.TriesToMove)
                    {
                        player.Entity.StopAnimation("liegen");
                        player.Entity.StartAnimation("gleiten");
                    }
                    else
                    {
                        player.Entity.StopAnimation("gleiten");
                        player.Entity.StartAnimation("liegen");
                    }

                    // InputEventListener - Disable actions
                    api.Input.InWorldAction += Input_InWorldAction;

                    if (player.Entity.CurrentControls == ((EnumEntityActivity)8 | (EnumEntityActivity)1))
                    {
                        api.Input.InWorldAction += Input_InWorldAction2;
                    }
                    else if (player.Entity.CurrentControls == ((EnumEntityActivity)8 | (EnumEntityActivity)2))
                    {
                        api.Input.InWorldAction += Input_InWorldAction2;
                    }
                    else if (player.Entity.CurrentControls == (EnumEntityActivity)2)
                    {
                        player.Entity.Controls.StopAllMovement();
                        api.Input.InWorldAction += Input_InWorldAction;
                    }
                    else if (player.Entity.CurrentControls == ((EnumEntityActivity)4 | (EnumEntityActivity)2))
                    {
                        player.Entity.Controls.StopAllMovement();
                        api.Input.InWorldAction += Input_InWorldAction;
                    }
                    else
                    {
                        api.Input.InWorldAction += Input_InWorldAction;
                    }
                }

                // Stop crawling
                if (aboveBlock == air || aboveBlock.IsLiquid())
                {
                    if ((api.Input.MouseButton.Middle == true || toggleStandup == true) && dashCountdown < 1 && player.Entity.CollidedVertically && isCrawling == true)
                    {
                        isCrawling = false;
                        toggleStandup = false;
                        player.Entity.StopAnimation("gleiten");
                        player.Entity.StopAnimation("liegen");
                        player.Entity.Properties.SetEyeHeight(1.7);

                        // InputEventListener
                        api.Input.InWorldAction += Input_InWorldAction1;
                    }
                }

                //Start Crawl
                if ((player.Entity.Controls.Sneak && api.Input.MouseButton.Middle == true || player.Entity.Controls.Sneak && toggleCrawl == true) && player.Entity.CollidedVertically && isCrawling == false)
                {
                    isCrawling = true;
                    toggleCrawl = false;
                }

                // Dash anticipation Animation
                if ((player.Entity.CurrentControls == ((EnumEntityActivity)4 | (EnumEntityActivity)2)) && player.Entity.Controls.TriesToMove && dashCountdown < 1 && stamina > drainStaminaDash)
                {
                    player.Entity.StartAnimation("awaitdash");
                }
                else
                {
                    player.Entity.StopAnimation("awaitdash");
                }

                // Reset dash animation
                if (dashCountdown <= maxDashCountdown-6)
                {
                    player.Entity.StopAnimation("dash");
                    player.Entity.StopAnimation("dashright");
                    player.Entity.StopAnimation("dashleft");
                }

                // Check if player is climbing for GUI
                if (player.Entity.Controls.IsClimbing)
                {
                    playerIsClimbing = true;
                }
                else
                {
                    playerIsClimbing = false;
                }

                // Dash start & animation
                if ((player.Entity.Controls.Sprint || player.Entity.Controls.Jump) && player.Entity.Controls.TriesToMove && player.Entity.Controls.RightMouseDown)
                {
                    if (stamina > drainStaminaDash && dashCountdown < 1 && !player.Entity.Controls.Backward && !player.Entity.Controls.Right && !player.Entity.Controls.Left)
                    {
                        player.Entity.StartAnimation("dash");
                        stamina = stamina -= drainStaminaDash;
                        dashCountdown = maxDashCountdown;
                        staminaRegenDelay = maxStaminaRegenDelay;
                        dashTime = 0;
                        motionStopedAfterDash = false;
                    }
                    else if (stamina > drainStaminaDash && dashCountdown < 1 && !player.Entity.Controls.Backward && player.Entity.Controls.Right)
                    {
                        player.Entity.StartAnimation("dashright");
                        stamina = stamina -= drainStaminaDash;
                        dashCountdown = maxDashCountdown;
                        staminaRegenDelay = maxStaminaRegenDelay;
                        dashTime = 0;
                        motionStopedAfterDash = false;
                    }
                    else if (stamina > drainStaminaDash && dashCountdown < 1 && !player.Entity.Controls.Backward && player.Entity.Controls.Left)
                    {
                        player.Entity.StartAnimation("dashleft");
                        stamina = stamina -= drainStaminaDash;
                        dashCountdown = maxDashCountdown;
                        staminaRegenDelay = maxStaminaRegenDelay;
                        dashTime = 0;
                        motionStopedAfterDash = false;
                    }
                    else if (stamina > drainStaminaDash && dashCountdown < 1 && player.Entity.Controls.Backward)
                    {
                        player.Entity.StartAnimation("backflip");
                        stamina = stamina -= drainStaminaDash;
                        dashCountdown = maxDashCountdown;
                        staminaRegenDelay = maxStaminaRegenDelay;
                        dashTime = 0;
                        motionStopedAfterDash = false;
                    }
                    else if (stamina < drainStaminaDash)
                    {
                        stamina = 0;
                        staminaRegenDelay = maxStaminaRegenDelay;
                    }
                }

                // Slide start
                if (player.Entity.Controls.Sprint && player.Entity.Controls.TriesToMove && (api.Input.MouseButton.Middle == true) && dashCountdown < 1 && !isCrawling)
                {

                    if (stamina > drainStaminaDash && dashCountdown < 1)
                    {
                        player.Entity.Controls.StopAllMovement();
                        isCrawling = true;
                        stamina = stamina -= drainStaminaDash;
                        dashCountdown = maxDashCountdown;
                        staminaRegenDelay = maxStaminaRegenDelay;
                        slideTime = 0;
                    }
                }

                // Slide-Movement /  50ms
                if (slideTime < 10)
                {
                    if (!player.Entity.CollidedVertically)
                    {
                        player.Entity.Pos.Motion = player.Entity.Pos.Motion * 1;
                        slideTime++;
                    }
                    else
                    {
                        player.Entity.Pos.Motion = player.Entity.Pos.Motion * 3;
                        slideTime++;
                    }
                }

                // Dash-Movement /  50ms
                if (dashTime < 5)
                {
                    if (!player.Entity.CollidedVertically && !player.Entity.Controls.Backward)
                    {
                        player.Entity.Pos.Motion = player.Entity.Pos.Motion * 1.10;
                        dashTime++;
                    }
                    else if (!player.Entity.CollidedVertically && player.Entity.Controls.Backward)
                    {
                        player.Entity.Pos.Motion = player.Entity.Pos.Motion * 1;
                        player.Entity.Pos.Motion.Y = player.Entity.Pos.Motion.Y * 0.6;
                        dashTime++;
                    }
                    else
                    {
                        player.Entity.Pos.Motion = player.Entity.Pos.Motion * 2;
                        dashTime++;
                    }
                }

                // Wall Jump
                if (player.Entity.Controls.IsClimbing && player.Entity.Controls.RightMouseDown && !player.Entity.Controls.Backward && player.Entity.Controls.Left)
                {
                    if (stamina > drainStaminaDash && dashCountdown < 1)
                    {
                        player.Entity.Properties.CanClimbAnywhere = false;          
                        player.Entity.Pos.Motion = player.Entity.Pos.Motion * 6;      
                        stamina = stamina -= drainStaminaDash;
                        staminaRegenDelay = maxStaminaRegenDelay;
                        dashCountdown = maxDashCountdown;

                        player.Entity.StartAnimation("dashleft");
                    }
                }
                else if (player.Entity.Controls.IsClimbing && player.Entity.Controls.RightMouseDown && !player.Entity.Controls.Backward && player.Entity.Controls.Right)
                {
                    if (stamina > drainStaminaDash && dashCountdown < 1)
                    {
                        player.Entity.Properties.CanClimbAnywhere = false;
                        player.Entity.Pos.Motion = player.Entity.Pos.Motion * 6;
                        stamina = stamina -= drainStaminaDash;
                        staminaRegenDelay = maxStaminaRegenDelay;
                        dashCountdown = maxDashCountdown;

                        player.Entity.StartAnimation("dashright");
                    }
                }
                else if (player.Entity.Controls.IsClimbing && player.Entity.Controls.RightMouseDown && player.Entity.Controls.Backward)
                {
                    if (stamina > drainStaminaDash && dashCountdown < 1)
                    {
                        player.Entity.Properties.CanClimbAnywhere = false;
                        player.Entity.Pos.Motion = player.Entity.Pos.Motion * 1.5;
                        stamina = stamina -= drainStaminaDash;
                        staminaRegenDelay = maxStaminaRegenDelay;
                        dashCountdown = maxDashCountdown;
                    }
                }

                // Charging jumpforce
                if (player.Entity.Controls.Sneak && player.Entity.CollidedVertically && !player.Entity.Controls.TriesToMove && !isCrawling)
                {
                    if (((maxChargeJump - chargeJump)/10 * drainStaminaChargeJump) < stamina + 1 && chargeJump <= maxChargeJump)
                    {
                        chargeJump += 10;
                        stamina -= drainStaminaChargeJump;
                    }
                }
                else if (chargeJump > 0)
                {
                    chargeJump -= 5;
                    if (chargeJump < 0)
                    {
                        chargeJump = 0;
                    }
                }

                // Charge Jump
                if (chargeJump >= 100)
                {
                    Vintagestory.API.Config.GlobalConstants.BaseJumpForce = 12f;
                    staminaRegenDelay = maxStaminaRegenDelay;
                    player.Entity.StartAnimation("powerjump");
                }
                else
                {
                    Vintagestory.API.Config.GlobalConstants.BaseJumpForce = 8.2f;
                    player.Entity.StopAnimation("powerjump");
                }

                // Check if block at playerPos is climable (ladder)
                if (checkBlockatPlayerPos.Climbable)
                {
                    isClimbable = true;
                }
                else
                {
                    isClimbable = false;
                }

                // Stamina drain while climbing
                if (player.Entity.Controls.IsClimbing && stamina > 0 && (player.Entity.Controls.TriesToMove || player.Entity.Controls.Sneak || player.Entity.Controls.Jump))
                {
                    float staminaNewValue = (stamina -= drainStaminaClimbing);
                    if (isClimbable == false)
                    {
                        if (staminaNewValue >= 0)
                        {
                            stamina = staminaNewValue;
                        }
                        else
                        {
                            stamina = 0;
                        }
                    }
                    else if (isClimbable == true)
                    {
                        stamina += drainStaminaClimbing;
                    }
                }
                else if (player.Entity.Controls.IsClimbing && stamina > 0 && !(player.Entity.Controls.TriesToMove || player.Entity.Controls.Sneak || player.Entity.Controls.Jump))
                {
                    float staminaNewValue = (stamina -= (drainStaminaClimbing * drainStaminaIdleModificator));
                    if (isClimbable == false)
                    {
                        if (staminaNewValue >= 0)
                        {
                            stamina = staminaNewValue;
                        }
                        else
                        {
                            stamina = 0;
                        }
                    }
                    else if (isClimbable == true)
                    {
                        stamina += (drainStaminaClimbing * drainStaminaIdleModificator);
                    }
                }

                // Ability to climb
                if (!player.Entity.Controls.IsClimbing)
                {
                    if (player.Entity.Controls.Sneak && stamina > drainStaminaClimbing && player.InventoryManager.ActiveHotbarSlot.Empty && (chargeJump < 1))
                    {
                        player.Entity.Properties.CanClimbAnywhere = true;
                    }
                    else
                    {
                        player.Entity.Properties.CanClimbAnywhere = false;
                    }
                }
                else
                {
                    if (stamina > 0 && player.InventoryManager.ActiveHotbarSlot.Empty)
                    {
                        player.Entity.Properties.CanClimbAnywhere = true;
                    }
                    else if (isClimbable == true)
                    {
                        player.Entity.Properties.CanClimbAnywhere = true;
                    }
                    else
                    {
                        player.Entity.Properties.CanClimbAnywhere = false;
                    }
                }

                // Stop crouch animation & allow movement while climbing (Crouch-climb fix)
                if (player.Entity.Controls.IsClimbing)
                {
                    player.Entity.StopAnimation("gleiten");
                    player.Entity.StopAnimation("liegen");
                    api.Input.InWorldAction += Input_InWorldAction1;
                }

                // Stepheight while sprinting
                if (player.Entity.Controls.Sprint)
                {
                    player.Entity.GetBehavior<EntityBehaviorControlledPhysics>().stepHeight = 1.0F;
                }
                else{
                    player.Entity.GetBehavior<EntityBehaviorControlledPhysics>().stepHeight = 0.9F;
                }

                //Delay Stamina Regen when Stamina < 1 (0)
                if (stamina < 1 && staminaRegenIsDelayed == false)
                {
                    staminaRegenDelay = maxStaminaRegenDelay;
                    staminaRegenIsDelayed = true;
                }

                // Stamina Regen
                if (stamina < maxStamina && !player.Entity.Controls.IsClimbing && player.Entity.CollidedVertically && !player.Entity.Controls.Sneak && staminaRegenDelay < 1)
                {
                    float staminaNewValue;
                    staminaNewValue = (stamina += staminaRegenValue);

                    if (staminaNewValue > maxStamina)
                    {
                        stamina = maxStamina;
                    }
                    else
                    {
                        stamina += staminaRegenValue;
                    }
                    
                    staminaRegenIsDelayed = false;
                }
            }, 50); //50 ms/tick


        }

        // Hotkey stand-up Method
        private bool StandUp(KeyCombination comb)
        {
            if (isCrawling)
            {
                MoveMod.toggleStandup = true;
                return true;
            }

            return true;
        }
        // Hotkey start-crawl Method
        private bool StartCrawl(KeyCombination comb)
        {
            if (!isCrawling)
            {
                MoveMod.toggleCrawl = true;
                return true;
            }

            return true;
        }

        // Eventhandler on Inventory-Gear changed
        //private void GearInventory_SlotModified(int t1)
        //{
        //    slotChanged = true;
        //}

        // EventHandler PlayerAction - Prevent Sneaking / climbing down
        private void Input_InWorldAction3(EnumEntityAction action, bool on, ref EnumHandling handled)
        {
            //Prevent sneak
            if (action == EnumEntityAction.Sneak)
            {
                handled = EnumHandling.PreventDefault;
            }
        }

        // EventHandler PlayerAction - Allow all movement & Prevent Sprint,Jump
        private void Input_InWorldAction2(EnumEntityAction action, bool on, ref EnumHandling handled)
        {
            //Prevent sprint / jump
            if (action == EnumEntityAction.Sprint)
            {
                handled = EnumHandling.PreventDefault;
            }
            else if (action == EnumEntityAction.Jump)
            {
                handled = EnumHandling.PreventDefault;
            }

            //allow all movement directions
            else if (action == EnumEntityAction.Forward)
            {
                handled = EnumHandling.PassThrough;
            }
            else if (action == EnumEntityAction.Left)
            {
                handled = EnumHandling.PassThrough;
            }
            if (action == EnumEntityAction.Right)
            {
                handled = EnumHandling.PassThrough;
            }
            else if (action == EnumEntityAction.Backward)
            {
                handled = EnumHandling.PassThrough;
            }
        }

        // EventHandler PlayerAction - Prevent all movement & Sprint,Jump
        private void Input_InWorldAction(EnumEntityAction action, bool on, ref EnumHandling handled)
        {
            //Prevent sprint / jump
            if (action == EnumEntityAction.Sprint)
            {
                handled = EnumHandling.PreventDefault;
            }
            else if (action == EnumEntityAction.Jump)
            {
                handled = EnumHandling.PreventDefault;
            }

            //prevent all movement directions
            if (action == EnumEntityAction.Forward)
            {
                handled = EnumHandling.PreventDefault;
            }
            else if (action == EnumEntityAction.Left)
            {
                handled = EnumHandling.PreventDefault;
            }
            if (action == EnumEntityAction.Right)
            {
                handled = EnumHandling.PreventDefault;
            }
            else if (action == EnumEntityAction.Backward)
            {
                handled = EnumHandling.PreventDefault;
            }

        }

        // EventHandler PlayerAction - Allow all movement
        private void Input_InWorldAction1(EnumEntityAction action, bool on, ref EnumHandling handled)
        {
            if (action == EnumEntityAction.Sprint)
            {
                handled = EnumHandling.PassThrough;
            }
            else if (action == EnumEntityAction.Jump)
            {
                handled = EnumHandling.PassThrough;
            }
            else if (action == EnumEntityAction.Sneak)
            {
                handled = EnumHandling.PassThrough;
            }

            //allow all movement directions
            else if (action == EnumEntityAction.Forward)
            {
                handled = EnumHandling.PassThrough;
            }
            else if (action == EnumEntityAction.Left)
            {
                handled = EnumHandling.PassThrough;
            }
            else if (action == EnumEntityAction.Right)
            {
                handled = EnumHandling.PassThrough;
            }
            else if (action == EnumEntityAction.Backward)
            {
                handled = EnumHandling.PassThrough;
            }
        }

    }

    // Stamina GUI
    // -----------------------------------------------------------------------------------
    public class HudOverlaySample : ModSystem
    {
        ICoreClientAPI capi;
        StaminaProgressBarRenderer renderer;

        public override bool ShouldLoad(EnumAppSide side)
        {
            return side == EnumAppSide.Client;
        }

        public override void StartClientSide(ICoreClientAPI api)
        {
            this.capi = api;
            renderer = new StaminaProgressBarRenderer(api);

            api.Event.RegisterRenderer(renderer, EnumRenderStage.Ortho);
        }
    }

    public class StaminaProgressBarRenderer : IRenderer
    {
        MeshRef whiteRectangleRef;
        MeshRef progressQuadRef;
        ICoreClientAPI capi;

        Matrixf mvMatrix = new Matrixf();

        public double RenderOrder { get { return 0; } }

        public int RenderRange { get { return 10; } }

        public StaminaProgressBarRenderer(ICoreClientAPI api)
        {
            this.capi = api;

            // This will get a line loop with vertices inside [-1,-1] till [1,1]
            MeshData rectangle = LineMeshUtil.GetRectangle(ColorUtil.WhiteArgb);
            whiteRectangleRef = api.Render.UploadMesh(rectangle);

            // This will get a quad with vertices inside [-1,-1] till [1,1]
            progressQuadRef = api.Render.UploadMesh(QuadMeshUtil.GetQuad());
        }

        public void OnRenderFrame(float deltaTime, EnumRenderStage stage)
        {
            IShaderProgram curShader = capi.Render.CurrentActiveShader;

            Vec4f color = new Vec4f(1, 1, 1, 1);

            if (MoveMod.playerIsClimbing == true || MoveMod.stamina < MoveMod.maxStamina)
            {
                // Render rectangle
                curShader.Uniform("rgbaIn", color);
                curShader.Uniform("extraGlow", 0);
                curShader.Uniform("applyColor", 0);
                curShader.Uniform("tex2d", 0);
                curShader.Uniform("noTexture", 1f);

                float width = 100;

                mvMatrix
                    .Set(capi.Render.CurrentModelviewMatrix)
                    .Translate(capi.Render.FrameWidth/2 - 50, capi.Render.FrameHeight/8, 0)
                    .Scale(width, 20, 0)
                    .Translate(0.5f, 0.5f, 0)
                    .Scale(0.5f, 0.5f, 0)
                ;     
                curShader.UniformMatrix("projectionMatrix", capi.Render.CurrentProjectionMatrix);
                curShader.UniformMatrix("modelViewMatrix", mvMatrix.Values);

                capi.Render.RenderMesh(whiteRectangleRef);
            }


            // Render progress bar
            if (MoveMod.playerIsClimbing == true || MoveMod.stamina < MoveMod.maxStamina)
            {
                if (MoveMod.stamina >= MoveMod.maxStamina)
                {
                    float width = 100;
                    mvMatrix
                        .Set(capi.Render.CurrentModelviewMatrix)
                        .Translate(capi.Render.FrameWidth/ 2 - 50, capi.Render.FrameHeight/8, 0)
                        .Scale(width, 20, 0)
                        .Translate(0.5f, 0.5f, 0)
                        .Scale(0.5f, 0.5f, 0)
                    ;
                }
                else
                {
                    float width = ((MoveMod.stamina) / MoveMod.maxStamina) * 100;
                    mvMatrix
                        .Set(capi.Render.CurrentModelviewMatrix)
                        .Translate(capi.Render.FrameWidth/2 - 50, capi.Render.FrameHeight / 8, 0)
                        .Scale(width, 20, 0)
                        .Translate(0.5f, 0.5f, 0)
                        .Scale(0.5f, 0.5f, 0)
                    ;
                }
            }

            if (MoveMod.playerIsClimbing == true || MoveMod.stamina < MoveMod.maxStamina)
            {
                curShader.UniformMatrix("projectionMatrix", capi.Render.CurrentProjectionMatrix);
                curShader.UniformMatrix("modelViewMatrix", mvMatrix.Values);

                capi.Render.RenderMesh(progressQuadRef);
            }

        }

        public void Dispose()
        {
                capi.Render.DeleteMesh(whiteRectangleRef);
                capi.Render.DeleteMesh(progressQuadRef);
        }
    }
}
